# Unix SMB/CIFS samba-tool-plus LDAP database module.

# Copyright (C) Paul Wolneykien <manowar@altlinux.org> 2017.
#
# Based on netcmd/user module by
#   Jelmer Vernooij <jelmer@samba.org>,
#   Matthias Dieter Wallnoefer,
#   Andrew Tridgell <tridge@samba.org> and
#   Giampaolo Lauria <lauria2@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Extended functions for using the SAM."""

import ldb
from samba.samdb import SamDB
from samba.common import normalise_int32

class SamDBPlus(SamDB):
    """The extended SAM database."""

    def usermod(self, username, password=None,
                force_password_change_at_next_login_req=False,
                useusernameascn=False, userou=None, surname=None,
                givenname=None, initials=None, profilepath=None,
                scriptpath=None, homedrive=None, homedirectory=None,
                jobtitle=None, department=None, company=None,
                description=None, mailaddress=None, internetaddress=None,
                telephonenumber=None, physicaldeliveryoffice=None, sd=None,
                setpassword=False, uidnumber=None, gidnumber=None, gecos=None,
                loginshell=None, uid=None, nisdomain=None, unixhome=None,
                smartcard_required=False):
        """Modifies user account parameters."""

        displayname = ""
        if givenname is not None:
            displayname += givenname

        if initials is not None:
            displayname += ' %s.' % initials

        if surname is not None:
            displayname += ' %s' % surname
        
        cn = username
        if useusernameascn is None and displayname is not "":
            cn = displayname
        user_dn = "CN=%s,%s,%s" % (cn, (userou or "CN=Users"), self.domain_dn())

        ldbmessage = ldb.Message()
        ldbmessage.dn = ldb.Dn(self, user_dn)

        def replace(name, val):
            if val is not None and val is not "":
                ldbmessage[name] = ldb.MessageElement(
                    val, ldb.FLAG_MOD_REPLACE, name)
            else:
                ldbmessage[name] = ldb.MessageElement(
                    [], ldb.FLAG_MOD_DELETE, name)
                    

        ### Modify the values
        
        if smartcard_required:
            replace("userAccountControl", str(dsdb.UF_NORMAL_ACCOUNT|dsdb.UF_SMARTCARD_REQUIRED))
            setpassword = False
        
        if surname is not None:
            replace("sn", surname)

        if givenname is not None:
            replace("givenName", givenname)

        if displayname is not "":
            replace("displayName", displayname)
            if useusernameascn is None:
                replace("name", displayname)

        if initials is not None:
            replace("initials", '%s.' % initials)

        if profilepath is not None:
            replace("profilePath", profilepath)

        if scriptpath is not None:
            replace("scriptPath", scriptpath)

        if homedrive is not None:
            replace("homeDrive", homedrive)

        if homedirectory is not None:
            replace("homeDirectory", homedirectory)

        if jobtitle is not None:
            replace("title", jobtitle)

        if department is not None:
            replace("department", department)

        if company is not None:
            replace("company", company)

        if description is not None:
            replace("description", description)

        if mailaddress is not None:
            replace("mail", mailaddress)

        if internetaddress is not None:
            replace("wWWHomePage", internetaddress)

        if telephonenumber is not None:
            replace("telephoneNumber", telephonenumber)

        if physicaldeliveryoffice is not None:
            replace("physicalDeliveryOfficeName", physicaldeliveryoffice)

        if sd is not None:
            replace("nTSecurityDescriptor", ndr_pack(sd))

        if uid is not None:
            replace("uid", str(uid))
            
        if uidnumber is not None:
            replace("uidNumber", str(uidnumber))

        if gidnumber is not None:
            replace("gidNumber", str(gidnumber))

        if gecos is not None:
            replace("gecos", str(gecos))

        if loginshell is not None:
            replace("loginShell", str(loginshell))

        if unixhome is not None:
            replace("unixHomeDirectory", str(unixhome))

        if nisdomain is not None:
            replace("msSFU30NisDomain", str(nisdomain))
            replace("msSFU30Name", str(username))
            # FIXME: The same thing is in the original SamDB module. WTF?
            replace("unixUserPassword", "ABCD!efgh12345$67890")
        
        self.transaction_start()
        try:
            self.modify(ldbmessage)
            if setpassword:
                self.setpassword(
                    "(samAccountName=%s)" % ldb.binary_encode(username),
                    password,
                    force_password_change_at_next_login_req)
        except:
            self.transaction_cancel()
            raise
        else:
            self.transaction_commit()

    def groupmod(self, groupname, groupou=None, grouptype=None,
                 description=None, mailaddress=None, notes=None,
                 sd=None, gidnumber=None, nisdomain=None):
        """Modifies group account parameters."""

        group_dn = "CN=%s,%s,%s" % (groupname, (groupou or "CN=Users"), self.domain_dn())

        # The new user record. Note the reliance on the SAMLDB module which
        # fills in the default informations

        ldbmessage = ldb.Message()
        ldbmessage.dn = ldb.Dn(self, group_dn)

        def replace(name, val):
            if val is not None and val is not "":
                ldbmessage[name] = ldb.MessageElement(
                    val, ldb.FLAG_MOD_REPLACE, name)
            else:
                ldbmessage[name] = ldb.MessageElement(
                    [], ldb.FLAG_MOD_DELETE, name)

        
        ### Modify the values
        
        if grouptype is not None:
            replace("groupType", normalise_int32(grouptype))

        if description is not None:
            replace("description", description)

        if mailaddress is not None:
            replace("mail", mailaddress)

        if notes is not None:
            replace("info", notes)

        if gidnumber is not None:
            replace("gidNumber", normalise_int32(gidnumber))

        if nisdomain is not None:
            replace("msSFU30Name", groupname)
            replace("msSFU30NisDomain", nisdomain)

        if sd is not None:
            replace("nTSecurityDescriptor", ndr_pack(sd))

        self.transaction_start()
        try:
            self.modify(ldbmessage)
        except:
            self.transaction_cancel()
            raise
        else:
            self.transaction_commit()
